import java.util.Vector;

public abstract class Transport {

    // Counts the number of transports instantiated
    private static int nextTransportID = 1;
    // Each such tranport gets a unique ID corresponding to its order of instantiation
    public final int transportID;

    protected final int seatedCapacity;
    protected final int standingCapacity;

    private Vector<Passenger> seatedPassengers = new Vector<Passenger>();
    private Vector<Passenger> standingPassengers = new Vector<Passenger>();

    protected Transport(int seats, int stands) {
        transportID = nextTransportID++;
        this.seatedCapacity = seats;
        this.standingCapacity = stands;
    }

    public void reachNextStop(String stopName) {
        System.out.println(this.getClass().getName() + " n°" + transportID + " has reached stop \"" + stopName + "\".");

        for (int i = 0; i < seatedPassengers.size(); i++)
            seatedPassengers.get(i).notifyNextStop();

        for (int i = 0; i < standingPassengers.size(); i++)
            standingPassengers.get(i).notifyNextStop();

        System.out.println(this.getClass().getName() + " n°" + transportID + " leaves with " + seatedPassengers.size() + "/" + seatedCapacity + " seated passengers, and "
                        + standingPassengers.size() + "/" + standingCapacity + " standing passengers.");
    }

    /*
     * If a passenger requests a seated place (i.e., wantSeat=true), and there are none left, then the pick up fails.
     * Similarly, if there are no standing place left, and wantSeat=false, the pick up fails.
     */
    protected boolean pickUpPassenger(Passenger p, boolean wantSeat) {
        if (wantSeat && seatedCapacity > seatedPassengers.size()) {
            seatedPassengers.add(p);
            return true;
        }

        if (!wantSeat && standingCapacity > standingPassengers.size()) {
            standingPassengers.add(p);
            return true;
        }

        // Passenger could not get on the Transport
        return false;
    }

    public int seatsAvailable() {
        return seatedCapacity - seatedPassengers.size();
    }

    public int standsAvailable() {
        return standingCapacity - standingPassengers.size();
    }

    /*
     * This method allows a Passenger to notify a Transport that he left
     */
    public void removePassenger(Passenger p1) {
        boolean wasThere = false;
        for (int i = 0; i < seatedPassengers.size(); i++) {
            if (p1 == seatedPassengers.get(i)) {
                seatedPassengers.remove(p1);
                wasThere = true;
                break;
            }
        }

        for (int i = 0; i < standingPassengers.size(); i++) {
            if (p1 == standingPassengers.get(i)) {
                standingPassengers.remove(p1);
                wasThere = true;
                break;
            }
        }

        if (wasThere)
            System.out.println(p1.fullName + " got off the " + this.getClass().getName() + " n°" + this.transportID + ".");
    }

}